function [TestData] = ReadData(FolderName, fn)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This function is used to rearrange data from rheometer              %%%
%%% experiments to convert it into a MATLAB structure. Note: the        %%%
%%% function only works with .csv files that have a comma as a decimal  %%%
%%% separator.                                                          %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Read the data and convert into a matrix
A = readmatrix(fullfile(FolderName,fn),"DecimalSeparator",".",'NumHeaderLines',0);
  
%% First Phase 
PC_comp        = A(1,3); %Phase Change Compression) Index at which the compression phase changes to the relaxation phase

A(1:4,:) = []; % Delete first 4 rows as these do not contain any relevant info.

PC_rlx    = A(PC_comp+1,3)+  PC_comp;  %Phase Change Relaxation. Index at which the relaxation phase changes to the tension phase

A(PC_comp+1:PC_comp+4,:) = []; %Delete rows between compression and relaxation phases

A(PC_comp+1:PC_rlx,2) = A(PC_comp+1:PC_rlx,2) + A(PC_comp,2); %connect index

%% Second Phase
A(PC_rlx+1:PC_rlx+5,:) = []; %Delete rows between compression and relaxation phases

A(:,1) = []; %Delete side column

%Connect indices and time 
A(PC_rlx+1:end,1) = A(PC_rlx+1:end,1)+A(PC_rlx,1); %index
A(PC_rlx+1:end,2) = A(PC_rlx+1:end,2)+A(PC_rlx,2); %time

%Define vector parameters
index       = A(:,1);
time        = A(:,2); 
NormalForce = A(:,3); 
velocity    = A(:,4);
gap         = A(:,5);
Temperature = A(:,7);

% Sometimes is happens that there is invalid data. This function checks if
% data is invalid; if so, gaps of invalid data are interpolated
[index, time, NormalForce, gap, Temperature] = InterpolateInvalidData(index, time, NormalForce, velocity, gap, Temperature);

% Define vector where the gap starts at 0 mm as soon as tensile force is applied:
index_positive_force = min(find(NormalForce<0));
index_positive_force = max([index_positive_force PC_rlx]);

%create a new vector that starts with 0 mm displacement
gap_shifted = gap - gap(index_positive_force);

%Sometimes the force is not accurately zeroed. The 'steady state force' is
%determined by the mean of the final 500 points. this force is subtracted
%from the measured force.
SteadyStateForce = mean(NormalForce(end-500:end));
NormalForce = NormalForce-SteadyStateForce;


%Output scalar input parameters
fs_compression  = 1/time(1);
fs_tension      = 1/((time(PC_comp+2)-time(PC_comp+1)));
F_compression   = str2double(fn(3));
t_compression   = str2double(fn(6:7));
v_tension       = -round(mean(velocity(PC_comp+1:end)));

%Rearrange all data into structure. 
TestData.index = index;
TestData.time = time;
TestData.NormalForce = NormalForce;
TestData.velocity = velocity;
TestData.gap = gap;
TestData.gap_shifted = gap_shifted;
TestData.fs_compression = fs_compression;
TestData.fs_tension = fs_tension;
TestData.F_compression = F_compression;
TestData.t_compression = t_compression;
TestData.v_tension = v_tension;
TestData.PC_comp = PC_comp;
TestData.PC_rlx = PC_rlx;
TestData.IndexPositiveForce = index_positive_force;
TestData.Temperature = Temperature;


end

